@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
@@ -64,7 +64,7 @@ pub trait PrincipalManager: Sync + Send {
         body: Option<Vec<u8>>,
     ) -> impl Future<Output = trc::Result<HttpResponse>> + Send;
 
-    fn assert_supported_directory(&self) -> trc::Result<()>;
+    fn assert_supported_directory(&self, override_: bool) -> trc::Result<()>;
 }
 
 impl PrincipalManager for Server {
@@ -75,8 +75,8 @@ impl PrincipalManager for Server {
         body: Option<Vec<u8>>,
         access_token: &AccessToken,
     ) -> trc::Result<HttpResponse> {
-        match (path.get(1), req.method()) {
-            (None, &Method::POST) => {
+        match (path.get(1).copied(), req.method()) {
+            (None | Some("deploy"), &Method::POST) => {
                 // Parse principal
                 let principal =
                     serde_json::from_slice::<PrincipalSet>(body.as_deref().unwrap_or_default())
@@ -99,7 +99,7 @@ impl PrincipalManager for Server {
                 })?;
 
                 // SPDX-SnippetBegin
-                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
                 // SPDX-License-Identifier: LicenseRef-SEL
 
                 #[cfg(feature = "enterprise")]
@@ -114,7 +114,7 @@ impl PrincipalManager for Server {
 
                 // Make sure the current directory supports updates
                 if matches!(principal.typ(), Type::Individual) {
-                    self.assert_supported_directory()?;
+                    self.assert_supported_directory(path.get(1).copied() == Some("deploy"))?;
                 }
 
                 // Validate roles
@@ -181,7 +181,7 @@ impl PrincipalManager for Server {
                 }
 
                 // Increment revision
-                self.increment_token_revision(result.changed_principals)
+                self.invalidate_principal_caches(result.changed_principals)
                     .await;
 
                 Ok(JsonResponse::new(json!({
@@ -253,7 +253,7 @@ impl PrincipalManager for Server {
                 }
 
                 // SPDX-SnippetBegin
-                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
                 // SPDX-License-Identifier: LicenseRef-SEL
 
                 let mut tenant = access_token.tenant.map(|t| t.id);
@@ -390,7 +390,7 @@ impl PrincipalManager for Server {
                             {
                                 Ok(changed_principals) => {
                                     // Increment revision
-                                    server.increment_token_revision(changed_principals).await;
+                                    server.invalidate_principal_caches(changed_principals).await;
                                 }
                                 Err(err) => {
                                     trc::error!(err.details("Failed to delete principal"));
@@ -445,7 +445,7 @@ impl PrincipalManager for Server {
                     .ok_or_else(|| not_found(name.to_string()))?;
 
                 // SPDX-SnippetBegin
-                // SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+                // SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
                 // SPDX-License-Identifier: LicenseRef-SEL
 
                 #[cfg(feature = "enterprise")]
@@ -539,7 +539,7 @@ impl PrincipalManager for Server {
                         }
 
                         // Increment revision
-                        self.increment_token_revision(changed_principals).await;
+                        self.invalidate_principal_caches(changed_principals).await;
 
                         Ok(JsonResponse::new(json!({
                             "data": (),
@@ -574,10 +574,8 @@ impl PrincipalManager for Server {
                         // Validate changes
                         for change in &changes {
                             match change.field {
-                                PrincipalField::Secrets => {
-                                    self.assert_supported_directory()?;
-                                }
-                                PrincipalField::Name
+                                PrincipalField::Secrets
+                                | PrincipalField::Name
                                 | PrincipalField::Emails
                                 | PrincipalField::Quota
                                 | PrincipalField::UsedQuota
@@ -588,7 +586,8 @@ impl PrincipalManager for Server {
                                 | PrincipalField::Members
                                 | PrincipalField::Lists
                                 | PrincipalField::Urls
-                                | PrincipalField::ExternalMembers => (),
+                                | PrincipalField::ExternalMembers
+                                | PrincipalField::Locale => (),
                                 PrincipalField::Tenant => {
                                     // Tenants are not allowed to change their tenantId
                                     if access_token.tenant.is_some() {
@@ -670,7 +669,7 @@ impl PrincipalManager for Server {
                             .await?;
 
                         // Increment revision
-                        self.increment_token_revision(changed_principals).await;
+                        self.invalidate_principal_caches(changed_principals).await;
 
                         Ok(JsonResponse::new(json!({
                             "data": (),
@@ -771,7 +770,7 @@ impl PrincipalManager for Server {
                         .await?;
 
                     // Increment revision
-                    self.increment_token_revision(ChangedPrincipals::from_change(
+                    self.invalidate_principal_caches(ChangedPrincipals::from_change(
                         access_token.primary_id(),
                         Type::Individual,
                         PrincipalField::Secrets,
@@ -793,7 +792,7 @@ impl PrincipalManager for Server {
         }
 
         // Make sure the current directory supports updates
-        self.assert_supported_directory()?;
+        self.assert_supported_directory(false)?;
 
         // Build actions
         let mut actions = Vec::with_capacity(requests.len());
@@ -842,7 +841,7 @@ impl PrincipalManager for Server {
             .await?;
 
         // Increment revision
-        self.increment_token_revision(changed_principals).await;
+        self.invalidate_principal_caches(changed_principals).await;
 
         Ok(JsonResponse::new(json!({
             "data": (),
@@ -850,7 +849,7 @@ impl PrincipalManager for Server {
         .into_http_response())
     }
 
-    fn assert_supported_directory(&self) -> trc::Result<()> {
+    fn assert_supported_directory(&self, override_: bool) -> trc::Result<()> {
         let class = match &self.core.storage.directory.store {
             DirectoryInner::Internal(_) => return Ok(()),
             DirectoryInner::Ldap(_) => "LDAP",
@@ -861,13 +860,17 @@ impl PrincipalManager for Server {
             DirectoryInner::OpenId(_) => "OpenID",
         };
 
-        Err(manage::unsupported(format!(
-            concat!(
-                "{} directory cannot be managed. ",
-                "Only internal directories support inserts ",
-                "and update operations."
-            ),
-            class
-        )))
+        if !override_ {
+            Err(manage::unsupported(format!(
+                concat!(
+                    "{} directory cannot be managed. ",
+                    "Only internal directories support inserts ",
+                    "and update operations."
+                ),
+                class
+            )))
+        } else {
+            Ok(())
+        }
     }
 }
