@@ -1,24 +1,32 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
-use std::{
-    borrow::Cow,
-    time::{Duration, Instant, SystemTime},
+use super::{ArcSeal, AuthResult, DkimSign};
+use crate::{
+    core::{Session, SessionAddress, State},
+    inbound::milter::Modification,
+    queue::{
+        self, Message, MessageSource, MessageWrapper, QueueEnvelope, Schedule, quota::HasQueueQuota,
+    },
+    reporting::analysis::AnalyzeReport,
+    scripts::ScriptResult,
 };
-
 use common::{
     config::{
-        smtp::{auth::VerifyStrategy, session::Stage},
+        smtp::{
+            auth::VerifyStrategy,
+            queue::{QueueExpiry, QueueName},
+            session::Stage,
+        },
         spamfilter::SpamFilterAction,
     },
     listener::SessionStream,
     psl,
     scripts::ScriptModification,
 };
-
 use mail_auth::{
     AuthenticatedMessage, AuthenticationResults, DkimResult, DmarcResult, ReceivedSpf,
     common::{headers::HeaderWriter, verify::VerifySignature},
@@ -30,20 +38,13 @@ use sieve::runtime::Variable;
 use smtp_proto::{
     MAIL_BY_RETURN, RCPT_NOTIFY_DELAY, RCPT_NOTIFY_FAILURE, RCPT_NOTIFY_NEVER, RCPT_NOTIFY_SUCCESS,
 };
-use store::write::now;
+use std::{
+    borrow::Cow,
+    time::{Instant, SystemTime},
+};
 use trc::SmtpEvent;
 use utils::config::Rate;
 
-use crate::{
-    core::{Session, SessionAddress, State},
-    inbound::milter::Modification,
-    queue::{self, Message, MessageSource, QueueEnvelope, Schedule, quota::HasQueueQuota},
-    reporting::analysis::AnalyzeReport,
-    scripts::ScriptResult,
-};
-
-use super::{ArcSeal, AuthResult, DkimSign};
-
 impl<T: SessionStream> Session<T> {
     pub async fn queue_message(&mut self) -> Cow<'static, [u8]> {
         // Parse message
@@ -604,7 +605,7 @@ impl<T: SessionStream> Session<T> {
             .unwrap_or(true)
         {
             headers.extend_from_slice(b"Return-Path: <");
-            headers.extend_from_slice(message.return_path.as_bytes());
+            headers.extend_from_slice(message.message.return_path.as_bytes());
             headers.extend_from_slice(b">\r\n");
         }
 
@@ -657,7 +658,7 @@ impl<T: SessionStream> Session<T> {
         }
 
         // Update size
-        message.size = (raw_message.len() + headers.len()) as u64;
+        message.message.size = (raw_message.len() + headers.len()) as u64;
 
         // Verify queue quota
         if self.server.has_quota(&mut message).await {
@@ -666,7 +667,9 @@ impl<T: SessionStream> Session<T> {
 
             // Queue message
             let source = if !self.is_authenticated() {
-                MessageSource::Unauthenticated
+                MessageSource::Unauthenticated(
+                    dmarc_result.is_some_and(|result| result == DmarcResult::Pass),
+                )
             } else {
                 MessageSource::Authenticated
             };
@@ -699,118 +702,31 @@ impl<T: SessionStream> Session<T> {
         mut rcpt_to: Vec<SessionAddress>,
         queue_id: u64,
         span_id: u64,
-    ) -> Message {
+    ) -> MessageWrapper {
         // Build message
         let created = SystemTime::now()
             .duration_since(SystemTime::UNIX_EPOCH)
             .map_or(0, |d| d.as_secs());
         let mut message = Message {
-            queue_id,
-            span_id,
             created,
-            return_path: mail_from.address,
-            return_path_lcase: mail_from.address_lcase,
-            return_path_domain: mail_from.domain,
+            return_path: mail_from.address_lcase,
             recipients: Vec::with_capacity(rcpt_to.len()),
-            domains: Vec::with_capacity(3),
             flags: mail_from.flags,
             priority: self.data.priority,
             size: 0,
             env_id: mail_from.dsn_info,
             blob_hash: Default::default(),
             quota_keys: Vec::new(),
+            received_from_ip: self.data.remote_ip,
+            received_via_port: self.data.local_port,
         };
 
         // Add recipients
-        let future_release = Duration::from_secs(self.data.future_release);
+        let future_release = self.data.future_release;
         rcpt_to.sort_unstable();
         for rcpt in rcpt_to {
-            if message
-                .domains
-                .last()
-                .is_none_or(|d| d.domain != rcpt.domain)
-            {
-                let rcpt_idx = message.domains.len();
-                message.domains.push(queue::Domain {
-                    retry: Schedule::now(),
-                    notify: Schedule::now(),
-                    expires: 0,
-                    status: queue::Status::Scheduled,
-                    domain: rcpt.domain,
-                });
-
-                let envelope = QueueEnvelope::new(&message, rcpt_idx);
-
-                // Set next retry time
-                let retry = if self.data.future_release == 0 {
-                    queue::Schedule::now()
-                } else {
-                    queue::Schedule::later(future_release)
-                };
-
-                // Set expiration and notification times
-                let config = &self.server.core.smtp.queue;
-                let (num_intervals, next_notify) = self
-                    .server
-                    .eval_if::<Vec<Duration>, _>(&config.notify, &envelope, self.data.session_id)
-                    .await
-                    .and_then(|v| (v.len(), v.into_iter().next()?).into())
-                    .unwrap_or_else(|| (1, Duration::from_secs(86400)));
-                let (notify, expires) = if self.data.delivery_by == 0 {
-                    (
-                        queue::Schedule::later(future_release + next_notify),
-                        now()
-                            + future_release.as_secs()
-                            + self
-                                .server
-                                .eval_if(&config.expire, &envelope, self.data.session_id)
-                                .await
-                                .unwrap_or_else(|| Duration::from_secs(5 * 86400))
-                                .as_secs(),
-                    )
-                } else if (message.flags & MAIL_BY_RETURN) != 0 {
-                    (
-                        queue::Schedule::later(future_release + next_notify),
-                        now() + self.data.delivery_by as u64,
-                    )
-                } else {
-                    let expire = self
-                        .server
-                        .eval_if(&config.expire, &envelope, self.data.session_id)
-                        .await
-                        .unwrap_or_else(|| Duration::from_secs(5 * 86400));
-                    let expire_secs = expire.as_secs();
-                    let notify = if self.data.delivery_by.is_positive() {
-                        let notify_at = self.data.delivery_by as u64;
-                        if expire_secs > notify_at {
-                            Duration::from_secs(notify_at)
-                        } else {
-                            next_notify
-                        }
-                    } else {
-                        let notify_at = -self.data.delivery_by as u64;
-                        if expire_secs > notify_at {
-                            Duration::from_secs(expire_secs - notify_at)
-                        } else {
-                            next_notify
-                        }
-                    };
-                    let mut notify = queue::Schedule::later(future_release + notify);
-                    notify.inner = (num_intervals - 1) as u32; // Disable further notification attempts
-
-                    (notify, now() + expire_secs)
-                };
-
-                // Update domain
-                let domain = message.domains.last_mut().unwrap();
-                domain.retry = retry;
-                domain.notify = notify;
-                domain.expires = expires;
-            }
-
             message.recipients.push(queue::Recipient {
-                address: rcpt.address,
-                address_lcase: rcpt.address_lcase,
+                address: rcpt.address_lcase,
                 status: queue::Status::Scheduled,
                 flags: if rcpt.flags
                     & (RCPT_NOTIFY_DELAY
@@ -823,11 +739,99 @@ impl<T: SessionStream> Session<T> {
                 } else {
                     rcpt.flags | RCPT_NOTIFY_DELAY | RCPT_NOTIFY_FAILURE
                 },
-                domain_idx: (message.domains.len() - 1) as u32,
                 orcpt: rcpt.dsn_info,
+                retry: Schedule::now(),
+                notify: Schedule::now(),
+                expires: QueueExpiry::Attempts(0),
+                queue: QueueName::default(),
             });
+
+            let envelope = QueueEnvelope::new(&message, message.recipients.last().unwrap());
+
+            // Set next retry time
+            let retry = if self.data.future_release == 0 {
+                queue::Schedule::now()
+            } else {
+                queue::Schedule::later(future_release)
+            };
+
+            // Resolve queue
+            let queue = self.server.get_queue_or_default(
+                &self
+                    .server
+                    .eval_if::<String, _>(
+                        &self.server.core.smtp.queue.queue,
+                        &envelope,
+                        self.data.session_id,
+                    )
+                    .await
+                    .unwrap_or_else(|| "default".to_string()),
+                self.data.session_id,
+            );
+
+            // Set expiration and notification times
+            let num_intervals = std::cmp::max(queue.notify.len(), 1);
+            let next_notify = queue.notify.first().copied().unwrap_or(86400);
+            let (notify, expires) = if self.data.delivery_by == 0 {
+                (
+                    queue::Schedule::later(future_release + next_notify),
+                    match queue.expiry {
+                        QueueExpiry::Ttl(time) => QueueExpiry::Ttl(future_release + time),
+                        QueueExpiry::Attempts(count) => QueueExpiry::Attempts(count),
+                    },
+                )
+            } else if (message.flags & MAIL_BY_RETURN) != 0 {
+                (
+                    queue::Schedule::later(future_release + next_notify),
+                    QueueExpiry::Ttl(self.data.delivery_by as u64),
+                )
+            } else {
+                let (notify, expires) = match queue.expiry {
+                    QueueExpiry::Ttl(expire_secs) => (
+                        (if self.data.delivery_by.is_positive() {
+                            let notify_at = self.data.delivery_by as u64;
+                            if expire_secs > notify_at {
+                                notify_at
+                            } else {
+                                next_notify
+                            }
+                        } else {
+                            let notify_at = -self.data.delivery_by as u64;
+                            if expire_secs > notify_at {
+                                expire_secs - notify_at
+                            } else {
+                                next_notify
+                            }
+                        }),
+                        QueueExpiry::Ttl(expire_secs),
+                    ),
+                    QueueExpiry::Attempts(_) => (
+                        next_notify,
+                        QueueExpiry::Ttl(self.data.delivery_by.unsigned_abs()),
+                    ),
+                };
+
+                let mut notify = queue::Schedule::later(future_release + notify);
+                notify.inner = (num_intervals - 1) as u32; // Disable further notification attempts
+
+                (notify, expires)
+            };
+
+            // Update recipient
+            let recipient = message.recipients.last_mut().unwrap();
+            recipient.retry = retry;
+            recipient.notify = notify;
+            recipient.expires = expires;
+            recipient.queue = queue.virtual_queue;
+        }
+
+        MessageWrapper {
+            queue_id,
+            queue_name: QueueName::default(),
+            is_multi_queue: false,
+            span_id,
+            message,
         }
-        message
     }
 
     pub async fn can_send_data(&mut self) -> Result<bool, ()> {
