@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: LicenseRef-SEL
  *
@@ -446,9 +446,9 @@ impl EventType {
             EventType::Eval(EvalEvent::Result) => 139,
             EventType::Eval(EvalEvent::StoreNotFound) => 140,
             EventType::TaskQueue(TaskQueueEvent::BlobNotFound) => 141,
-            EventType::TaskQueue(TaskQueueEvent::Index) => 142,
-            EventType::TaskQueue(TaskQueueEvent::BayesTrain) => 143,
-            EventType::TaskQueue(TaskQueueEvent::Locked) => 144,
+            EventType::MessageIngest(MessageIngestEvent::FtsIndex) => 142,
+            EventType::Spam(SpamEvent::TrainAccount) => 143,
+            EventType::TaskQueue(TaskQueueEvent::TaskLocked) => 144,
             EventType::TaskQueue(TaskQueueEvent::MetadataNotFound) => 145,
             EventType::Housekeeper(HousekeeperEvent::Run) => 146,
             EventType::Housekeeper(HousekeeperEvent::Schedule) => 149,
@@ -766,7 +766,7 @@ impl EventType {
             EventType::Smtp(SmtpEvent::RcptToMissing) => 466,
             EventType::Smtp(SmtpEvent::RcptToRewritten) => 467,
             EventType::Smtp(SmtpEvent::RelayNotAllowed) => 468,
-            EventType::Smtp(SmtpEvent::RemoteIdNotFound) => 469,
+            EventType::Smtp(SmtpEvent::IdNotFound) => 469,
             EventType::Smtp(SmtpEvent::RequestTooLarge) => 470,
             EventType::Smtp(SmtpEvent::RequireTlsDisabled) => 471,
             EventType::Smtp(SmtpEvent::Rset) => 472,
@@ -887,6 +887,14 @@ impl EventType {
             EventType::Store(StoreEvent::CacheHit) => 51,
             EventType::Store(StoreEvent::CacheStale) => 52,
             EventType::Store(StoreEvent::CacheUpdate) => 577,
+            EventType::TaskQueue(TaskQueueEvent::TaskAcquired) => 578,
+            EventType::Calendar(CalendarEvent::AlarmSent) => 579,
+            EventType::Calendar(CalendarEvent::AlarmSkipped) => 580,
+            EventType::Calendar(CalendarEvent::AlarmRecipientOverride) => 581,
+            EventType::Calendar(CalendarEvent::AlarmFailed) => 582,
+            EventType::Calendar(CalendarEvent::ItipMessageSent) => 583,
+            EventType::Calendar(CalendarEvent::ItipMessageReceived) => 584,
+            EventType::Calendar(CalendarEvent::ItipMessageError) => 585,
         }
     }
 
@@ -1029,9 +1037,9 @@ impl EventType {
             139 => Some(EventType::Eval(EvalEvent::Result)),
             140 => Some(EventType::Eval(EvalEvent::StoreNotFound)),
             141 => Some(EventType::TaskQueue(TaskQueueEvent::BlobNotFound)),
-            142 => Some(EventType::TaskQueue(TaskQueueEvent::Index)),
-            143 => Some(EventType::TaskQueue(TaskQueueEvent::BayesTrain)),
-            144 => Some(EventType::TaskQueue(TaskQueueEvent::Locked)),
+            142 => Some(EventType::MessageIngest(MessageIngestEvent::FtsIndex)),
+            143 => Some(EventType::Spam(SpamEvent::TrainAccount)),
+            144 => Some(EventType::TaskQueue(TaskQueueEvent::TaskLocked)),
             145 => Some(EventType::TaskQueue(TaskQueueEvent::MetadataNotFound)),
             146 => Some(EventType::Housekeeper(HousekeeperEvent::Run)),
             149 => Some(EventType::Housekeeper(HousekeeperEvent::Schedule)),
@@ -1385,7 +1393,7 @@ impl EventType {
             466 => Some(EventType::Smtp(SmtpEvent::RcptToMissing)),
             467 => Some(EventType::Smtp(SmtpEvent::RcptToRewritten)),
             468 => Some(EventType::Smtp(SmtpEvent::RelayNotAllowed)),
-            469 => Some(EventType::Smtp(SmtpEvent::RemoteIdNotFound)),
+            469 => Some(EventType::Smtp(SmtpEvent::IdNotFound)),
             470 => Some(EventType::Smtp(SmtpEvent::RequestTooLarge)),
             471 => Some(EventType::Smtp(SmtpEvent::RequireTlsDisabled)),
             472 => Some(EventType::Smtp(SmtpEvent::Rset)),
@@ -1510,6 +1518,14 @@ impl EventType {
             51 => Some(EventType::Store(StoreEvent::CacheHit)),
             52 => Some(EventType::Store(StoreEvent::CacheStale)),
             577 => Some(EventType::Store(StoreEvent::CacheUpdate)),
+            578 => Some(EventType::TaskQueue(TaskQueueEvent::TaskAcquired)),
+            579 => Some(EventType::Calendar(CalendarEvent::AlarmSent)),
+            580 => Some(EventType::Calendar(CalendarEvent::AlarmSkipped)),
+            581 => Some(EventType::Calendar(CalendarEvent::AlarmRecipientOverride)),
+            582 => Some(EventType::Calendar(CalendarEvent::AlarmFailed)),
+            583 => Some(EventType::Calendar(CalendarEvent::ItipMessageSent)),
+            584 => Some(EventType::Calendar(CalendarEvent::ItipMessageReceived)),
+            585 => Some(EventType::Calendar(CalendarEvent::ItipMessageError)),
             _ => None,
         }
     }
@@ -1583,6 +1599,7 @@ impl Key {
             Key::ValidTo => 62,
             Key::Value => 63,
             Key::Version => 64,
+            Key::QueueName => 65,
         }
     }
 
@@ -1653,6 +1670,7 @@ impl Key {
             62 => Some(Key::ValidTo),
             63 => Some(Key::Value),
             64 => Some(Key::Version),
+            65 => Some(Key::QueueName),
             _ => None,
         }
     }
