@@ -1,9 +1,12 @@
 /*
- * SPDX-FileCopyrightText: 2020 Stalwart Labs Ltd <hello@stalw.art>
+ * SPDX-FileCopyrightText: 2020 Stalwart Labs LLC <hello@stalw.art>
  *
  * SPDX-License-Identifier: AGPL-3.0-only OR LicenseRef-SEL
  */
 
+use dav_proto::Depth;
+use hyper::StatusCode;
+
 use super::WebDavTest;
 
 pub async fn test(test: &WebDavTest) {
@@ -17,8 +20,8 @@ pub async fn test(test: &WebDavTest) {
         .with_header(
             "dav",
             concat!(
-                "1, 2, 3, access-control, extended-mkcol, ",
-                "calendar-access, calendar-no-timezone, addressbook"
+                "1, 2, 3, access-control, extended-mkcol, calendar-access, ",
+                "calendar-auto-schedule, calendar-no-timezone, addressbook"
             ),
         )
         .with_header(
@@ -43,6 +46,30 @@ pub async fn test(test: &WebDavTest) {
             ["/dav/cal/", "/dav/cal/jane/", "/dav/cal/support/"],
         );
 
+    // Test 404 responses
+    jane.sync_collection(
+        "/dav/cal/jane/default/",
+        "",
+        Depth::Infinity,
+        None,
+        ["D:getetag"],
+    )
+    .await;
+    jane.sync_collection(
+        "/dav/cal/jane/test-404/",
+        "",
+        Depth::Infinity,
+        None,
+        ["D:getetag"],
+    )
+    .await;
+    jane.request("PROPFIND", "/dav/cal/jane/default/", "")
+        .await
+        .with_status(StatusCode::MULTI_STATUS);
+    jane.request("PROPFIND", "/dav/cal/jane/test-404/", "")
+        .await
+        .with_status(StatusCode::NOT_FOUND);
+
     john.delete_default_containers().await;
     jane.delete_default_containers().await;
     jane.delete_default_containers_by_account("support").await;
